/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.ProgressIndicator");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.core.internal.util");

dojo.declare("net.jazz.ajax.ui.ProgressIndicator", dijit._Widget, {
	
	// summary:
	//		a widget for displaying progress messages
	// description:
	//		This widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application. To style it,
	//		define a class with the selector "net-jazz-ajax-ProgressIndicator"
	//		and add	your desired rules.
	// since:
	//		0.6
	
	//
	// Internal methods; do not call directly
	//
	
	create: function(args, srcNodeRef) {
		if(!srcNodeRef) {
			srcNodeRef = document.createElement("div");
		}
		// subscribe to events
		
		net.jazz.ajax.ui.ProgressIndicator.superclass.create.call(this, args, srcNodeRef);
	},
	
	postCreate: function() {
		dojo.addClass(this.domNode, "net-jazz-ajax-ProgressIndicator");
		// subscribe to relevant events
		dojo.subscribe(net.jazz.ajax.internal.events.showMessage, this, "_showMessage");
		dojo.subscribe(net.jazz.ajax.internal.events.hideMessageArea, this, "_hideMessageArea");
		// create animations for fade-in and fade-out
		var args = {node: this.domNode, duration: 250};
		this.fadeInAnimation = dojo.fadeIn(args);
		this.fadeOutAnimation = dojo.fadeOut(args);
		
		dojo.connect(this.fadeOutAnimation,"onEnd",this,function() {
			this.domNode.style.display = "none";
		});
		
		// hide the node before first use
		this.domNode.style.display = "none";
	},
	
	_showMessage: function(/*String*/ msg) {
		if(!this.domNode.parentNode) {
			return;
		}
		this.domNode.style.display = "block";
		net.jazz.ajax.core.internal.util.removeChildren(this.domNode);
		this.domNode.appendChild(document.createTextNode(msg + "..."));
		this.fadeInAnimation.play();
	},
	
	_hideMessageArea: function() {
		if(this.domNode.parentNode){
			this.fadeOutAnimation.play();
		}
	}
	
});