/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "TeamContributorSelectionDialogMessages");

(function() {

var DELAY = 500;
var KEY_ESCAPE = 27;
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var KEY_SPACE = 32;
var MAX_INT = 2147483647;
var SEARCH_LIMIT = 250;

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
dojo.declare("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/TeamContributorSelectionDialog.html"),
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "TeamContributorSelectionDialogMessages");
		this.showingFirstMsg = dojo.string.substitute(this.messages.showingFirst, [ SEARCH_LIMIT ]);
		if (args == null) {
			args = {};
		}
		this.width = args.width || "450px";
		var properties = {
			"dialogTitle" : args.allowMultiple == null || args.allowMultiple ? this.messages.selectUsers : this.messages.selectUser,
			"width" : "450px",
			"closable": true,
			"hideAdminGuest": false,
			"allowArchivedUsers": true,
			"removeUsersOnAdd": false,
			"disableUsersOnAdd": true,
			"filterOutUsers" : [ ],
			"disableUsers" : [ ],
			"searchLimit" : SEARCH_LIMIT,
			"allowMultiple" : true,
			"useOkCancel" : false
		};
		for (var prop in properties) {
			if (args[prop] != null) {
				this[prop] = args[prop];
			}
			else {
				this[prop] = properties[prop];
			}
		}
		this.selectedUsers = [];
	},
	
	postCreate: function () {
		this.searchCount = 0;
		this.userSelector.size = 15;
		this.keyStrokeFilter = dojo.connect(document, "onkeypress", this, function (e) {
			if (e.keyCode === 8 && e.target !== this.searchText) {
				dojo.stopEvent(e);
			}
		});
		if (this.useOkCancel) {
			this.addCloseButton.appendChild(document.createTextNode(this.messages.ok));
			this.closeButton.appendChild(document.createTextNode(this.messages.cancel));
			this.addCloseButton.style.display = "";
			this.closeButton.style.display = "";
		}
		else {
			this.addCloseButton.appendChild(document.createTextNode(this.messages.addAndClose));
			this.addButton.appendChild(document.createTextNode(this.messages.add));
			this.closeButton.appendChild(document.createTextNode(this.messages.close));
			this.addCloseButton.style.display = "";
			this.addButton.style.display = "";
			this.closeButton.style.display = "";
		}
		this.userSelector.multiple = this.allowMultiple;
		if (!this.allowArchivedUsers) {
			this.filtersDiv.style.display = this.allowArchivedUsers ? "" : "none";
		}
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: this.width,
			closable: this.closable
		});
		this.connect(this.searchText,"onkeypress","handleSearchChange");
		this.connect(this.searchText,"onchange","handleSearchChange");
		this.connect(this.showAllLink,"onclick", function (e) {
			dojo.stopEvent(e);
			this.searchText.value = "*";
			this._doSearch();
		});
		this.connect(this.closeButton,"onclick", function (e) {
			dojo.stopEvent(e);
			this.dialog.close();
		});
		this.connect(this.addCloseButton,"onclick","_onAddClose");
		this.connect(this.addButton,"onclick","_onAdd");
		this.connect(this.userSelector,"ondblclick","_handleDoubleClick");
		this.connect(this.userSelector,"onkeydown","_handleKeyDown");
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.connect(this.userSelector,"onchange", function(e) {
				var selectedIndices = [];
				var disabledSelected = false;
				if (this.allowMultiple) {
					for (var i=0; i < this.userSelector.options.length; i++) {
						if (this.userSelector.options[i].selected) {
							if (this.userSelector.options[i].disabled) {
								disabledSelected = true;
								this.userSelector.options[i].selected = false;
							}
							else {
								selectedIndices.push(i);
							}
						}
					}
					if (disabledSelected) {
						if (selectedIndices.length > 0) {
							this.userSelector.savedSelected = selectedIndices;
						}
						else {
							for (var i=0; i < this.userSelector.savedSelected.length; i++) {
								this.userSelector.options[this.userSelector.savedSelected[i]].selected = true;
							}
						}
					}
					else {
						this.userSelector.savedSelected = selectedIndices;
					}
				}
				else {
					if (this.userSelector.selectedIndex >= 0) {
						if (this.userSelector.options[this.userSelector.selectedIndex].disabled) {
							if (this.userSelector.savedSelectedIndex >= 0) {
								this.userSelector.selectedIndex = this.userSelector.savedSelectedIndex;
							}
							else {
								this.userSelector.selectedIndex = -1;
							}
						}
						else {
							this.userSelector.savedSelectedIndex = this.userSelector.selectedIndex;
						}
					}
				}
			});
		}
		this.connect(this.userSelector,"onchange", function () {
			var selected = false;
			this.selectedUsers = [];
			for (var i=0; i < this.userSelector.options.length; i++) {
				if (this.userSelector.options[i].selected) {
					selected = true;
					this.selectedUsers.push({
						itemId: this.userSelector.options[i].itemId,
						name: this.userSelector.options[i].userName,
						userId: this.userSelector.options[i].userId,
						emailAddress: this.userSelector.options[i].emailAddress,
						archived: this.userSelector.options[i].archived
					});
				}
			}
			this.addCloseButton.disabled = !selected;
			this.addButton.disabled = !selected;
		});
		this.connect(this.filterArchivedCheckbox,"onclick","_doSearch");
		this.connect(this.dialog,"onClose","onClose");
		this.searchText.focus();
		if (this.disableUsersWithLicense) {
			var srh = new ServiceResponseHandler(this, "_disableUsersCallback", "_disableUsersErrorCallback");
			AdminClient.getContributors({searchTerm: this.disableUsersWithLicense, isLicenseSearch: true, pageSize: MAX_INT, hideGuest: true, hideUnassigned: true, pageNum: 0}, srh);
		}
	},
	
	_disableUsersCallback: function (resultSet) {
		if (resultSet != null && resultSet.elements != null) {
			for (var i=0; i < resultSet.elements.length; i++) {
				this.disableUsers.push({ "itemId": resultSet.elements[i].itemId });
			}
		}
	},
	
	_disableUsersErrorCallback: function (err) { /* noop */ },
	
	_handleDoubleClick: function (e) {
		if (this.selectedUsers.length > 0) {
			this._onAddClose(e);
		}
	},
	
	_handleKeyDown: function (e) {
		if ((e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) && this.selectedUsers.length > 0) {
			this._onAddClose(e);
		}
	},
	
	handleSearchChange: function (e) {
		if(e.keyCode == KEY_ESCAPE) {
			return;
		}
		if (e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) {
			if (this.selectedUsers.length == 0 && this.userSelector.options.length > 0) {
				var index = 0;
				while (index < this.userSelector.options.length && this.userSelector.options[index].disabled) {
					index++;
				}
				if (index != this.userSelector.options.length) {
					this.userSelector.options[index].selected = true;
					this.selectedUsers.push({
						itemId: this.userSelector.options[index].itemId,
						name: this.userSelector.options[index].userName,
						userId: this.userSelector.options[index].userId,
						emailAddress: this.userSelector.options[index].emailAddress,
						archived: this.userSelector.options[index].archived
					});
				}
			}
			if (this.selectedUsers.length > 0) {
				this._onAddClose(e);
			}
		}
		else {
			this._doCancellableSearch();
		}
	},
	
	_onAddClose: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUsers);
		this.dialog.close();
	},
	
	_onAdd: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUsers);
	},
	
	// an equivalent to onAdd that can be connected to.
	onOK: function (selectedUsers) {},
	
	onAdd: function (selectedUsers) {
		this.selectedUsers = [];
		this.addCloseButton.disabled = true;
		this.addButton.disabled = true;
		for (var i=0; i < this.userSelector.options.length; i++) {
			if (this.userSelector.options[i].selected) {
				this.userSelector.options[i].selected = false;
				if (this.removeUsersOnAdd) {
					this.filterOutUsers.push({'itemId': this.userSelector.options[i].itemId});
					if (dojo.isIE) {
						// IE doesn't handle removeChild and insertBefore well for select lists
						// Do things the old-fashioned way
						var option = this.userSelector.options[i];
						var opt = option;
						for (var j=i; j < this.userSelector.options.length - 1; j++) {
							var curOpt = this.userSelector.options[j+1];
							var newOpt = new Option(curOpt.text, curOpt.value);
							newOpt.itemId = curOpt.itemId;
							newOpt.name = curOpt.name;
							newOpt.userId = curOpt.userId;
							newOpt.emailAddress = curOpt.emailAddress;
							newOpt.archived = curOpt.archived;
							this.userSelector.options[j] = newOpt;
						}
						this.userSelector.options.length = this.userSelector.options.length - 1;
					}
					else {
						this.userSelector.removeChild(this.userSelector.options[i]);
					}
				}
				else if (this.disableUsersOnAdd) {
					this.disableUsers.push({'itemId': this.userSelector.options[i].itemId});
					if (dojo.isIE) {
						this.userSelector.options[i].style.color = "gray";
						if (this.userSelector.savedSelected != null && this.userSelector.savedSelected.length > 0) {
							this.userSelector.savedSelected = [];
						}
					}
					this.userSelector.options[i].disabled = "disabled";
				}
			}
		}
		this.onOK(selectedUsers);
	},
	
	onClose: function () {
		dojo.disconnect(this.keyStrokeFilter);
	},
	
	_doCancellableSearch: function() {
		if (this._currentId) {
			clearTimeout(this._currentId);
		}
		this._currentId= setTimeout(dojo.hitch(this, function () {this._doSearch();}), DELAY);
	},
	
	/*
	 * Common name patterns that include international characters
	 */
	_nameList: [ 
			[ /andre/, "andr?" ],
			[ /Andre/, "Andr?" ],
			[ /ohl[a-z][a-z]/, "ohl??" ],
			[ /Ohl[a-z][a-z]/, "Ohl??" ],
			[ /ohl[a-z]/, "ohl?" ],
			[ /Ohl[a-z]/, "Ohl?" ],
			[ /krug/, "kr?g" ],
			[ /Krug/, "Kr?g" ],
			[ /hogl/, "h?gl" ],
			[ /Hogl/, "H?gl" ]
	],
	
	_translateValue: function(value) {
		var result = value;
		dojo.forEach(this._nameList, function(namePair) {
			result = result.replace(namePair[0], namePair[1]);
		});
		return result
	},
	
	_doSearch: function () {
		// if there's no input, do nothing
		var value = this.searchText.value;
		if (dojo.trim(value) === "") {
			this._clearStatusMessage();
			return;
		}
		try{ 
			value = this._translateValue(value);
		} catch(e) {};
		var params;

		value = value.replace(/\*/g,"%");
		value = value.replace(/\?/g,"_");
		params = {
			sortBy: "name",
			searchTerm: value,
			searchField: "name",
			pageSize: this.searchLimit,
			hideAdminGuest: this.hideAdminGuest,
			hideUnassigned: true,
			hideArchivedUsers: !this.allowArchivedUsers || !this.filterArchivedCheckbox.checked,
			filterOutUsers: this.filterOutUsers
		};
		
		if (this.latestSearchTerm != null && this.latestSearchTerm === value) {
			// don't re-run the same search twice
			return;
		}
		this.latestSearchTerm = value;

		this._setStatusMessage(this.messages.runningSearch);
		this.searchCount++;
		var closureSearchCount = this.searchCount;
		var self = this;
		var serviceObj = {
			success: function (obj) {
				if (closureSearchCount === self.searchCount) {
					self._loadQueryResults(obj);
				}
			},
			failure: function (errorObj) {
				self._queryError(obj);
			}
		}
		var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
		this.runContribSearch(params, srh);
	},
	
	runContribSearch: function (params, srh) {
		AdminClient.getContributors(params, srh);
	},
	
	_loadQueryResults: function (resultSet) {
		var disabledItemIdHash = {};
		var disabledUserIdHash = {};
		for (var i=0; i < this.disableUsers.length; i++) {
			if (this.disableUsers[i].itemId != null) {
				disabledItemIdHash[this.disableUsers[i].itemId] = true;
			}
			else if (this.disableUsers[i].userId != null) {
				disabledUserIdHash[this.disableUsers[i].userId] = true;
			}
		}
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.userSelector.savedSelected = [];
		}
		if (dojo.isIE < 7) {
			this.userSelector.options.length = 0;
		}
		else {
			dojox.data.dom.removeChildren(this.userSelector);
		}
		if (resultSet != null && resultSet.elements != null) {
			this.addCloseButton.disabled = true;
			this.addButton.disabled = true;
			for (var i=0; i < resultSet.elements.length; i++) {
				var disp = this.getDispValue(resultSet.elements[i]);
				var option = new Option(disp, resultSet.elements[i].itemId);
				option.itemId = resultSet.elements[i].itemId;
				option.userName = resultSet.elements[i].name;
				option.userId = resultSet.elements[i].userId;
				option.emailAddress = resultSet.elements[i].emailAddress;
				option.archived = resultSet.elements[i].archived;
				if (disabledItemIdHash[option.itemId] || disabledUserIdHash[option.userId]) {
					option.disabled = "disabled";
					// compensate for the IE bug that prevents us from disabling options
					if (dojo.isIE) {
						option.style.color = "gray";
					}
				}
				if (dojo.isIE) {
					this.userSelector.options[this.userSelector.options.length] = option;
				}
				else {
					this.userSelector.appendChild(option);
				}
			}
			if (resultSet.elements.length === SEARCH_LIMIT) {
				this.searchLimitMsg.style.display = "block";
			}
			else {
				this.searchLimitMsg.style.display = "none";
			}
		}
		else {
			this.addCloseButton.disabled = true;
			this.addButton.disabled = true;
		}
		this._clearStatusMessage();
	},
	
	getDispValue: function (elem) {
		return elem.name;
	},
	
	_queryError: function (err) {
		this._setStatusMessage(err.message, true);
	},
	
	_clearStatusMessage: function() {
		this.searchStatus.style.visibility = "hidden";
	},
	
	_setStatusMessage: function(msg, isError) {
		this.searchStatus.style.visibility = "visible";
		dojox.data.dom.replaceChildren(this.searchStatus, document.createTextNode(msg));
		this.searchStatus.className= isError ? "ErrorStatus" : "NormalStatus";
	}
});

})();