/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardApplication");

dojo.require("net.jazz.ajax.ui._Application");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.repository.web.ui.wizard.Wizard");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardIntro");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.EnableEmail");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.SetupUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");

function workbenchRootNode(){
	return net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
}

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigureDatabase = com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase;
var EnableEmail = com.ibm.team.repository.web.ui.internal.wizard.EnableEmail;
var SetupUsers = com.ibm.team.repository.web.ui.internal.wizard.SetupUsers;
var Wizard = com.ibm.team.repository.web.ui.wizard.Wizard;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var Intro = com.ibm.team.repository.web.ui.internal.wizard.WizardIntro;
var Review = com.ibm.team.repository.web.ui.internal.wizard.WizardReview;
var ConfigurePublicURI = com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI;
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;
var teamServer = "TEAM SERVER";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardApplication", net.jazz.ajax.ui._Application, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
		this.getUserRoles();
	},	

	_startApp: function () {
		this.invokeServer();
		document.body.style.background = "none";
		this._createUI();
		this.setName(this.messages.setupName);
		this.start();
	},
	
	_createUI: function() {		
		dojo.addClass(workbenchRootNode(), "com-ibm-team-repository-web-setup-RootNode");
		this.content = document.createElement("div");
		dojo.addClass(this.content, "com-ibm-team-repository-wizard");
		
		var textHeader = document.createElement("div");
		textHeader.innerHTML= dojo.string.substitute(this.messages.setupHeading, [teamServer]);
		textHeader.className = "setup-header"
		this.content.appendChild(textHeader);
		
		var jazzIcon = document.createElement("img");
		jazzIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/jazz-identR.png");
		jazzIcon.className = "jazz-icon";
		this.content.appendChild(jazzIcon);
		
		var setupGraphic = document.createElement("img");
		setupGraphic.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/JTS-wizard-graphic.png");
		setupGraphic.className = "wizard-graphic";
		this.content.appendChild(setupGraphic);

		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);		
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			processFinish:  function(){
				document.location.href = net.jazz.ajax._contextRoot + "/admin";
            }
		};
		
		if (this.authorized) { 
			wizardParams.steps = this.initServerSetupSteps();
		} else {
			var notAuthStep = new NotAuthorizedView({applicationName: this.messages.setupName});
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		var wizard = new Wizard(wizardParams);
		
		var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		
		
		workbenchRootNode().appendChild(this.content);
		
	},
	
	initServerSetupSteps: function () {
		var steps = [
			new Intro(),
			new ConfigureDatabase(),
			new EnableEmail(),
			new ConfigurePublicURI(),
			new SetupUsers()
		]
		if (window.location.search.indexOf("ref") == -1) {
			steps.push(new Review());
		}
		return steps;
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function(errorObject) {
		this._startApp();
	},
	
	invokeServer: function () {
		// To be invoked asynchronously to activate the services on a newly started server
		var srh= new ServiceResponseHandler(this, "doNothing", "doNothing");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.repository.service.internal.MailerService");
	}, 
	
	doNothing: function () {
		// do nothing
	}
});
var messages;
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});

})();