/*******************************************************************************
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.wizard", "WizardConfigurationStepMessages");


(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;

var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var STATUS_TYPES = com.ibm.team.repository.web.ui.wizard.WizardStepHeader.STATUS_TYPES;

dojo.declare("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep",  [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/wizard/templates/WizardConfigurationStep.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.wizard", "WizardConfigurationStepMessages");
		this.service = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
		this.pendingNext = null;
		this.enableNext = false;
		this.showTestButton = true;
		this.showRestoreButton = true;
		this.configUtil = new ConfigUtility();
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle, helpContent: this.helpContent});
		this.stepHeaderArea.appendChild(this.header.domNode);
		if(this.description) {
			if(isNode(this.description)) {
				var node = this.description;
			} else {
				node = document.createTextNode(this.description);
			}
			this.descriptionArea.appendChild(node);
		}
		if(!this.showTestButton) {
			this._testConnectionButton.style.display = "none";
		}
		if(!this.showRestoreButton) {
			this._revertButton.style.display = "none";
		}
		this._setState(this.STATUS.LOADING);
		this.createInnerStepUI();
	},
	
	createInnerStepUI: function () {
		if (this.innerArgs) {
			var innerStep1 = new InnerStep(this.innerArgs);
			this.serviceContainer = innerStep1.boxContents;
			this.configArea.appendChild(innerStep1.domNode);
		}
	},
	
	focusStep: function () {
		if (!this._config) {
			this._initializeContent();
		}
	},
	
	_initializeContent: function () {
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: this.implementationClassName
			},
			enableEdits: true,
			serviceHeaderClass: ["noDisplay"], 
			displayIn: this.serviceContainer,
			descriptionMode: "table", 
			initialMode: "edit"
		}
		if (this.extraServiceConfigArgs){
			dojo.mixin(serviceConfigProps, this.extraServiceConfigArgs)
		}

		this._config = new ServerPropertyService(serviceConfigProps);
		dojo.connect(this._config, "_show", this, "_initializePropertiesAfterLoaded");	
	},

	_initializePropertiesAfterLoaded: function () {
		dojo.connect(this._config, "_markChanged", this, function() {
			this._setState(this.STATUS.PENDING_OK);
		});
		dojo.connect(this._config, "_checkUnchanged", this, function () {
				if(this._config._changed) {
					this._setState(this.STATUS.PENDING_OK);
				} else {
					this._setState(this._initialLoadedState);
				}
			});
		dojo.connect(this._config, "_markInvalid", this, "_handleInvalidInput");
		this._testConnection();
	},	
	
	canFlipToNextPage: function () {
		return this.enableNext === true;
	},
	
	onNext: function() {
		this.invokeSave(this, "processNext", "saveError");
	},
		
	invokeSave: function(caller, onSuccess, onFailure) {
		var changeList = this._getChanges()
		if (changeList.changes.length > 0) {
			this.pending = changeList;
			this.configUtil.save(changeList, caller, onSuccess, onFailure);
			if (this.wizard) {
				this.wizard.updateNavigationControls();
			}
		} else {
			caller[onSuccess].call(caller, []);
		} 
	},
		
	processNext: function() {
		this._initialLoadedState = this.STATUS.CONNECTED_OK;
		if (this.pending){
			var pendingProperties = this.pending.properties;
			for (var i = 0; i < pendingProperties.length; i++) {
				if (pendingProperties[i]._restartRequired) {
					serverRestart = true;
				}
				pendingProperties[i]._handleSuccessfulChange();
			}
		} 
		// TODO handle server restart message by passing in a "pending restart" state to setState
		this._setState(this.STATUS.CONNECTED_OK);
		this.pending = null;
		this.wizard.updateNavigationControls();
		this.wizard.processNext();
	},
	
	saveError: function(errorObj) {	
		this._setState(this.STATUS.SAVE_ERR, errorObj, /*errorPath*/ true);
	},		
	
	// HANDLE USER INTERACTION METHODS
	 onInnerButtonAreaClick: function(e) {
		var button = e.target;
		if (button === this._revertButton) {
			this._revert();
		} else if (button === this._testConnectionButton){
			this._testConnection();
		} 
	}, 
		
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this._setState(this.STATUS.PENDING_ERR);
		} else {
			this._setState(this.STATUS.PENDING_WARN);
		}
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		// need to clear onkeyup timeout on every element
		var props = this._config._properties;
		for (var i=0; i < props.length; i++) {
			if(props[i].onKeyUpTimout){
				clearTimeout(props[i].onKeyUpTimout);
			}
		};	
		
		this._setState(this.STATUS.TESTING);
		var srh = new ServiceResponseHandler(this, "_testConnectionCallback", "_testConnectionError");
		this._config.testRobustValidity(srh);
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
	
		if (msg != null) {
			connected = this.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = this.STATUS.CONNECTED_ERR;
				}
			}
		} else {
			connected = this.STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected, msg);
	},
	
	_testConnectionError: function(errorObj) {
		this._setState(this.STATUS.CONNECTED_ERR, errorObj, /*errorPath*/ true);
	},
	
	_revert: function() {
		this._config.restoreSaved();
		this._setState(this._initialLoadedState);
	},

	// HANDLE STATE METHODS
	_setState: function(newState, details, fromErrorCallback) {
		var state = newState;
		// check required for initial load before service properties are loaded
		if (this._config != null) {
			this._allowPropertyEdit(state.edit); 
		}
		var enableRevert = state.revert;
		if (enableRevert && this._config && !this._config._changed) {
			enableRevert = false;
		}
		this._revertButton.disabled = (!enableRevert);
		this._testConnectionButton.disabled = (!state.test);
		this.enableNext = state.next;
		if (this.wizard) {
			this.wizard.updateNavigationControls();
		}
		if (state.code === this.STATUS.CONNECTED_OK.code && this.finalStep()) {
			this.STATUS.CONNECTED_OK.message = "connectedOkFinish";
		}
		var messageString = state.resolvedMessage ? state.resolvedMessage : messages[state.message]
		if (fromErrorCallback) {
			this._showErrorStatus(state.code, messageString , details);
		}
		else {
			this._showConnectionStatus(state.code, messageString, details);
		}
		
	},
				
	_allowPropertyEdit: function (allowEdits){
		var props = this._config._properties;
		for (var i=0; i < props.length; i++) {
			props[i]._editValueNode.disabled = !allowEdits;
		};	 
	},
		
	_showConnectionStatus: function(type, summary, details) {
		if (type != "NONE") {
			this.header.showStatus({type: type,	summary: summary, details: details})
		} else {
			this.header.clearMessages();
		}
		this._setStatusIndication(type, summary);
	}, 
	
	_showErrorStatus: function(type, summary, errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj, summary);
		this._setStatusIndication(type, summary);
	}, 
	
	_setStatusIndication: function (type, summary) {
		if (type != "INFO" && type != "NONE") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (type != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}
		} else {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		}
	}, 
			
	// SUPER ConfigUtility METHODS	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		if (this._config._changed) {
			var changeObj = this._config._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		return {
			"changes": changes,
			"properties": properties
		};
	}, 
	
	STATUS: {
		LOADING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS",
			message: 'loading'},  
		TESTING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS", 
			message: 'testing'},  
		DISABLED: {test: false, revert: true, next: true, edit: true, code: "OK", 
			message: 'disabled'},
		PENDING_OK: {test: true, revert: true, next: false, edit: true, code: "INFO", 
			message: 'pendingOk'},  
		PENDING_ERR: {test: false, revert: true, next: false, edit: true, code: "ERROR",
			message: 'pendingErr'}, 
		PENDING_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
			message: 'pendingWarn'}, 
		CONNECTED_OK: {test: true, revert: true, next: true, edit: true, code: "OK",  
			message: 'connectedOk'},
		CONNECTED_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
			message: 'connectedErr'}, 
		CONNECTED_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
			message: 'connectedWarn'}, 
		SAVE_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
			message: 'saveErr'}
	}
		
});


dojo.declare("com.ibm.team.repository.web.ui.wizard.InnerStep", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/wizard/templates/InnerStep.html")

});

var InnerStep = com.ibm.team.repository.web.ui.wizard.InnerStep;


function isNode(o){
	return o && o.nodeType && o.cloneNode;
}
})();